#! /usr/bin/perl

## Aufgabe von POP3auth.pl ist es, eine POP3-authentifizierte SMTP-bertragung
## zu initiieren. Dazu wird lediglich eine normale POP3-Verbindung hergestellt.
## Durch den speziellen Type von Eintrag in der VxMail-Datenbank wei der POP3-
## Server (VxPOP) dann, da er die E-Mails des Folders per SMTP an den Rechner
## mit der aus der POP3-Verbindung ermittelten IP-Nummer bertragen mu.
## Die aufgebaute POP3-Verbindung kann (theoretisch) direkt nach erfolgreicher
## Anmeldung wieder beendet werden; es ist jedoch manchmal zweckmig, diese
## ber einen gewissen Zeitraum hinweg aufrecht zu erhalten - um sicherzugehen,
## da z.B. ein ISDN-Router auf der Empfangsseite die Verbindung nicht
## vorzeitig unterbricht. Dazu kann mit der Option `-t' (`--timeout') ein
## Zeitwert (in Sekunden) angegeben werden, auf dessen Ablauf dieses Skript
## wartet, bevor es die POP3-Verbindung beendet; hierbei wird ca. jede Sekunde
## ein `NOOP'-Befehl bertragen, um die Verbindung am Leben zu erhalten.
## In einer spteren Version des POP3-Servers werde ich zustzlich vorsehen,
## da dieser im Fall einer erfolgreich beendeten SMTP-bertragung selbstttig
## meldet, da die Verbindung beendet werden kann, indem der `NOOP'-Befehl in
## diesem Fall mit einer Fehlermeldung zurckkehrt.

## POP3-Konfigurationsdaten

$SERVER = '<your servername>';
$USER = '<your username>';
$PASS = '<your password>';

# AB DIESEM PUNKT BITTE NICHTS MEHR NDERN

use Cwd;
use Socket;
use IO::Handle;

sub rfc822_date (@);
sub pop3_answer (*);
sub pop3_result (*);
sub pop3_ts (*);
sub pop3_quit (*$);
sub usage (@);

($progname = $0) =~ s(.*/)();
($progpath = $0) =~ s(\Q$progname\E$)();
$progpath =~ s(^[.]/)(cwd () . "/$1")e;
$progpath =~ s(^([^./])|^$)(cwd () . "/$1")e;
$progpath =~ s(/$)();
$filler = ' ' x (length $progname);

$silent = 1;
$configure = 0;
$timeout = 0;
while (@ARGV > 0 && ($_ = $ARGV[0]) =~ /^-/) {
    shift;
    last if (/^--$/);
    if (/^-c$|^--reconfigure$/) {
	$configure = 1;
    } elsif (/^-h$|^-help$|^--help$/) {
	usage ();
    } elsif (/^-v$|^--verbose$/) {
	$silent = 0;
    } elsif (/^-t(.*)|^--timeout=(.+)|^--timeout$/) {
	unless ($1) {
	    unless (@ARGV) { usage ("missing option argument for `$_'!\n"); }
	    $timeout = $1 + 0;
	} else {
	    $timeout = $1;
	}
    } else {
	usage ("unknown option `$_'!");
    }
}

if ($configure) { configure (@ARGV); exit 0; }

unless ($port = getservbyname ('pop3', 'tcp')) { $port = 110; }
$proto = getprotobyname ('tcp');
$iaddr = inet_aton ($SERVER);
$paddr = sockaddr_in ($port, $iaddr);

socket (SOCK, PF_INET, SOCK_STREAM, $proto) || die "$progname(socket): $!\r\n";
connect (SOCK, $paddr) || die "$progname (connect): $!\r\n";
SOCK->autoflush (1);

$md5ts = pop3_ts (SOCK);
die "$progname(MD5TS): $!\r\n" unless (defined $md5ts);
if ($md5ts) {
    $md5dig = md5digest ("$md5ts$PASS");
    print SOCK "APOP $USER $md5dig\r\n";
    print "APOP $USER $md5dig\r\n" unless ($silent);
    if (pop3_answer (SOCK)) { goto APOP_OK; }
    pop3_answer (SOCK);
}
print SOCK "USER $USER\r\n";
print "USER $USER\r\n" unless ($silent);
unless (pop3_answer (SOCK)) { pop_quit (SOCK, "invalid user $USER"); }
print SOCK "PASS $PASS\r\n";
print "PASS $PASS\r\n" unless ($silent);
unless (pop3_answer (SOCK)) { pop_quit (SOCK, "invalid password"); }

APOP_OK:

print SOCK "STAT\r\n";
print "STAT\r\n" unless ($silent);
$answer = pop3_result (SOCK);
unless (defined $answer) { die "$progname: POP3 seems not functional!\r\n"; }
($nmsgs, $fsize) = pop3_status ($answer);

while ($timeout > 0) {
    sleep 1;
    print SOCK "NOOP\r\n";
    print "NOOP\r\n" unless ($silent);
    $answer = pop3_result (SOCK);
    unless (defined $answer) {
	die "$progname: POP3 connection terminated!\r\n";
    }
    --$timeout;
}
    

print SOCK "QUIT\r\n";
print "QUIT\r\n" unless ($silent);
unless (pop3_answer (SOCK)) {
    print STDERR "$progname(POP3): QUIT cmd failed - i will nonetheless ",
		 "close\r\n$filler        the connection.\r\n",
}
close SOCK;

print "+++ $progname complete +++\r\n" unless ($silent);
exit 0;

sub pop3_answer (*) {
    my $SOCK = shift;
    local $_;
    $_ = <$SOCK>;
    print "$_" unless ($silent);
    (/^\+/ ? 1 : (/^\-/ ? 0 : undef))
}

sub pop3_result (*) {
    my $SOCK = shift;
    local $_;
    $_ = <$SOCK>;
    print "$_" unless ($silent);
    s/\r\n$//;
    (/^\+/ ? (substr $_, 4) : (/^\-/ ? '' : undef))
}

sub md5digest ($) {
    local $_ = `echo -n '$_[0]' | md5sum`;
    chomp; s/\s*-\s*$//;
    print "MD5-Digest = $_\r\n" unless ($silent);
    $_
}

sub pop3_ts (*) {
    my $SOCK = (shift);
    local $_;
    $_ = <$SOCK>;
    print "$_" unless ($silent);
    s/\r\n$//;
    if (/^\+.*(<.+>)/) {
	$1
    } else {
	(/^\+/ ? (substr $_, 4) : (/^\-/ ? '' : undef))
    }
}

sub pop3_status ($) {
    local $_ = (shift);
    s/^\s+//; s/\s+$//;
    print "$_\r\n" unless ($silent);
    split /\s+/, $_
}

sub pop_quit (*$) {
    my ($SOCK, $msg) = @_;
    print SOCK "QUIT\r\n";
    print "QUIT\r\n" unless ($silent);
    close SOCK;
    die "$progname(POP3): $msg\r\n";
}

sub usage (@) {
    my $li;
    if (@_) {
	$li = (shift);
	print STDERR "$progname: $li\r\n";
	for $li (@_) { print STDERR "$filler  $li\r\n"; }
    }
    die "Usage: $progname [-t <timeout>] [-q]\r\n",
    "       $progname -c [SERVER=<server>] [USER=<user>] [PASS=<passwd>]\r\n",
	"       $progname -h\r\n",
	"\r\n  Options:",
	"\r\n    -t (alt: --timeout)",
	"\r\n       wait <timeout> seconds before terminating the POP3 ",
	"connection",
	"\r\n    -q (alt: --quiet, -s, --silent)",
	"\r\n        turn off verbose output (more than one `-q' makes",
	"\r\n        `$progname' even more silent)",
	"\r\n    -h (alt: -help, --help)",
	"\r\n        print this help text",
	"\r\n    -c (alt: --configure)",
	"\r\n        re-configure this program\r\n";
}

sub configure (@) {
    my ($srv, $usr, $pwd);
    my ($el, @lines, $cr);
    for $el (@_) {
	if ($el =~ /^SERVER=(.+)$/) {
	    $srv = $1;
	} elsif ($el =~ /^USER=(.+)$/) {
	    $usr = $1;
	} elsif ($el =~ /^PASS=(.+)$/) {
	    $pwd = $1;
	}
    }
    open PROG, "<$0" or die "$progname: configuation failed - $!\r\n";
    @lines = map { chomp; if (s/\r$//) { $cr = 1; } $_ } <PROG>;
    close PROG;
    for $el (@lines) {
	if ($el =~ /^\$SERVER\s*=/ && (defined $srv)) {
	    if ($srv =~ /\'/) {
		$el = '$SERVER = "' . $srv . '";';
		$el =~ s/\\/\\\\/;
		$el =~ s/\$/\\\$/;
		$el =~ s/\@/\\\@/;
	    } else {
		$el = '$SERVER = ' . "'$srv';";
	    }
	} elsif ($el =~ /^\$USER\s*=/ && (defined $usr)) {
	    if ($usr =~ /\'/) {
		$el = '$USER = "' . $usr . '";';
		$el =~ s/\\/\\\\/;
		$el =~ s/\$/\\\$/;
		$el =~ s/\@/\\\@/;
	    } else {
		$el = '$USER = ' . "'$usr';";
	    }
	} elsif ($el =~ /^\$PASS\s*=/ && (defined $pwd)) {
	    if ($pwd =~ /\'/) {
		$el = '$PASS = "' . $pwd . '";';
		$el =~ s/\\/\\\\/;
		$el =~ s/\$/\\\$/;
		$el =~ s/\@/\\\@/;
	    } else {
		$el = '$PASS = ' . "'$pwd';";
	    }
	}
    }
    open PROG, ">$0" or die "$progname: configuration failed - $!\r\n";
    map { printf PROG "%s%s\n", $_, ($cr ? "\r" : ""); } @lines;
    close PROG;
}
