/* fparser.h
**
** Interface for `fparser.c'
** (Parser for an e-mail filter)
**
*/
#ifndef _FPARSER_H
#define _FPARSER_H

#include "regex/regex.h"

#include "filter.h"

typedef enum {
    ac_none,
    ac_insert,		/* insert a new header element... */
    ac_replace,		/* replace a header element ... */
    ac_delete,		/* delete a header element ... */
    ac_mailto,		/* send a mail to someone ... */
    ac_forward,		/* forward the message to someone ... */
    ac_drop,		/* drop mail ... */
    ac_exec,		/* apply an external filter program to the e-mail ... */
    ac_mailbox		/* insert the mail into a mailbox ... */
} acmode_t;
typedef struct action_s *action_t;
typedef struct condlist_s *condlist_t;

struct filter_s {
    filter_t next;
    int head, body;		/* head/body - auf Kopf/Rumpf anzuwenden */
    condlist_t conditions;	/* ODER-verknpfte Bedigungen */
    int result;			/* <>/= 0 => bereits/noch nicht angewendet */
    int pre;			/* Pr-Filter */
    action_t action;		/* Liste der Aktionen. */
};

struct condlist_s {
    condlist_t next;
    condlist_t also;
    int alsores;
    regex_t rx;
};

struct action_s {
    action_t next;
    acmode_t action;
    union {
	struct { char *elem; int ix; char *new; } insert;
	struct { char *elem; int ix; char *new; } replace;
	struct { char *elem; int ix; } delete;
	struct { char *from, *to, *msg; } mailto;
	struct { char *from, *to; } forward;
	struct { char *from; } drop;
	struct { char *from, *cmd; } exec;
	struct { char *from; char *name; } mailbox;
    } a;
};

filter_t fparser (FILE *fp, char *vars[], const char *sender);
void filter_free (filter_t filter);
const char *actype (action_t ac);

#endif /* defined(_FPARSER_H) */
