/* xfile.h
**
** Interface file for xfile.c
** (Handle folder files with index files)
**
*/
#ifndef _XFILE_H
#define _XFILE_H
struct __xFile;

typedef struct __xFile *xFile;

/*#include <stdio.h>*/
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

/* Gre des Lese-/Schreibpuffers einstellen */
extern void xSetBufSize (size_t siz);

/* ffnen eines Daten-/Indexdateipaares. Die Anzahl der Parameter ist von
** `amode' abhngig:
**    amode = `r'	=>	keine zus. Parameter
**    amode = `w'	=>	Zugriffsbits
**    amode = `u'	=>	Zugriffsbits
**    amode = `i'	=>	Zugriffsbits
**    amode = `we'	=>	Prefixlnge, Indexlnge, Zugriffsbits
**    amode = `ue'	=>	Prefixlnge, Indexlnge, Zugriffsbits
**    amode = `ie'	=>	Prefixlnge, Indexlnge, Zugriffsbits
**    amode = `re'	=>	dummy, dummy
** Zustzlich kann `amode' auch noch entweder `s' (setzen einer Sperre fr
** mehrfachen Zugriff) oder `x' (setzen einer Sperre fr ausschlielichen
** Zugriff) enthalten.
** Prefixlnge und Indexlnge sind Grenangaben fr Daten, die zustzlich
** in der Indexdatei untergebracht werden sollen (z.B. Daten zur Verwaltung
** von IMAP4-artigen eindeutigen Identifikationsnummern).
*/
extern xFile xOpen (char *path, char *amode, ...);

/* Setze den Dateizeiger auf den Datensatz mit der Nummer `index'. */
extern int xSeek (xFile fx, off_t index);

/* Schreibe den Pufferinhalt zurck in die Datendatei. */
extern ssize_t xFlush (xFile fx);

/* Schreibe Daten in die Datendatei */
extern ssize_t xWrite (xFile fx, void *buf, size_t bfs);

/* Lies Daten aus der Datendatei */
extern ssize_t xRead (xFile fx, void *buf, size_t bfs);

/* Gib die Lnge des Eintrages der Datendatei mit der Nummer `pos' zurck */
extern ssize_t xMsgLen (xFile fx, int pos);

/* Beende den aktuellen Eintrag in der Datendatei; d.h. schreibe die
** zustzlichen Daten zusammen mit der Endeposition in die Indexdatei.
** Falls die Datei mit amode = `i' geffnet wurde, dann mu als weiterer
** Parameter die gewnschte Dateiposition mitangegeben werden, da in diesem
** Modue lediglich die Indexdatei neu geschrieben wird.
*/
extern int xSet (xFile fx, ...);

/* Im Lesemodus (amode = `r'):     Setze den Dateizeiger auf den Anfang des
**                                 aktuellen Datensatzes zurck.
** Im Schreib-/Update-/Indexmodus: Beende den aktuellen Eintrag in der
**                                 Datendatei (nur Schreib-/Updatemodus),
**                                 setze den Datezeiger auf den Anfang des
**                                 ersten Eintrages in der Datendatei und
**                                 schalte in den Lesemodus um.
*/
extern int xReSet (xFile fx);

/* Im Lesemodus (amode = `r'):     Setze den Dateizeiger ans Ende der Daten
**                                 bzw. Indexdatei, leere den Schreib-/Lese-
**                                 puffer und erlaube wieder einen schreibenden
**                                 Umgang mit der Folders. Ergebnis ist `0'.
** Im Schreib-/Update-/Indexmodus: Tue nichts und kehre mit `-1' zurck.
*/
int xSetWrite (xFile fx);

/* Schneide die `xFile'-Dateien ab den dem angegebenen Index entsprechenden Positionen
** ab. Diese Positionen werden durch einen Aufruf von `xoffset' bestimmt.
** Auerdem werden die `xFile'-internen Dateizeiger auf die dem Index entsprechenden
** Positionen zurckgesetzt (mittels `xSeek').
*/
extern int xTruncate (xFile fx, int index);

/* Im Schreib-/Update-/Lesemodus:
**      Leere Daten- und Indexdatei und schalte in den Updatemodus um.
** Fr den Indexmodus (amode = `i') liefert diese Operation einen Fehler.
*/
extern int xReWrite (xFile fx);

/* Im Schreib-/Updatemodus:
**      Schreibe den aktuellen Pufferinhalt zurck und beende den aktuellen
**      Datensatz.
** Schliee (danach) die Daten- und Indexdatei und gib die durch das `xFile'-
** Handle belegten Resourcen wieder frei.
*/
extern int xClose (xFile fx);

/* Schreibe eine ASCII-NUL-terminierte Zeichenkette in die Datendatei. */
extern ssize_t xPuts (char *buf, xFile fx);

/* Lies ein einzelnes Zeichen aus dem aktuellen Datensatz der Datendatei. */
extern int xGetc (xFile fx);

/* Lies eine Textzeile aus dem aktuellen Datensatz der Datendatei. */
extern ssize_t xGets (char *buf, size_t siz, xFile fx);

/* Schreibe ein einzelnes Zeichen in die Datendatei. */
extern int xPutc (int cc, xFile fx);

/* Lsche Daten- und Indexdatei. */
extern int xUnlink (char *file);

/* Lsche nur die Indexdatei. */
extern int xNoIndex (char *file);

/* Stelle fest, ob sich Datenstze in der Datendatei befinden (Ergebnis = 0)
** oder nicht (Ergebnis = 1).
*/
extern int xEmpty (xFile fx);

/* xIsNew (filehandle)
**     gibt 1 zurck, falls die Indexdatei vllig neu erzeugt wurde und
**     0 sonst. Sobald das erste mal ein xSet aufgerufen wurde, ist das
**     Ergebnis 0.
*/
extern int xIsNew (xFile fx);

/* ndere den Besitzer und die Gruppe der Daten- und Indexdatei. */
extern int xChown (xFile fs, uid_t uid, gid_t gid);

/* ndere die Zugriffsbits der Daten- und Indexdatei. */
extern int xChmod (xFile fx, int mode);

/* Lies (maximal) `sz' Bytes zustzlicher Daten aus der Indexdatei. */
extern size_t xGetExtra (xFile fx, void *buf, size_t sz);

/* Schreibe (maximal) `sz' Bytes zustzlicher Daten in die Indexdatei.
** Diese Daten werden erst beim nchsten Aufruf von `xSet' in die Indexdatei
** bertragen.
*/
extern size_t xSetExtra (xFile fx, void *buf, size_t sz);

/* Lies (maximal) `sz' Bytes Prefix-Daten aus der Indexdatei. */
extern size_t xGetPrefix (xFile fx, void *buf, size_t sz);

/* Schreibe (maximal) `sz' Bytes Prefix-Daten in die Indexdatei.
** Diese Daten werden erst beim nchsten Aufruf von `xSet' in die Indexdatei
** bertragen.
*/
extern size_t xSetPrefix (xFile fx, void *buf, size_t sz);

/* Gib den Modus zurck, mit dem die Datei geffnet wurde */
extern char xAmode (xFile fx);

/* Kopiere `sz' Bytes aus den Prefix-Daten von `fx' in die Prefix-Daten von
** `fy'.
*/
extern void xCopyPrefix (xFile fx, xFile fy, size_t sz);

/* Kopiere `sz' Bytes aus den Indexdaten von `fx' in die Indexdaten von `fy'. */
extern void xCopyExtra (xFile fx, xFile fy, size_t sz);

/* Fhre `fstat' auf der geffneten Datendatei aus. */
extern int xStat (xFile fx, struct stat *buf);

/* Existenztest fr die Index-/Datendatei eines xFile. */
int xCheck (char *file);

/* xSize (fx) - gibt die aktuelle Lnge der Folderdatei zurck; hierbei werden
**              evtl. noch in den Puffern stehende Daten nicht bercksichtigt.
*/
extern off_t xSize (xFile fx);

/* xChown (fx, owner, group)
**     - ndert Benutzer- und Gruppenkennung der Daten- und Indexdateien eines
**       `xFile' auf die Werte `owner' und `group'.
*/
int xChown (xFile fx, uid_t owner, gid_t group);

/* xRename (old, new)
**     - benennt sowohl Daten- als auch Indexdatei von `old' zu `new' um.
*/
int xRename (char *old, char *new);

/* xMsgNo (fx)
**     - liefert im Lesemodus die Positionsnummer der aktuellen E-Mail.
*/
int xMsgNo (xFile fx);

#endif /* defined(_XFILE_H) */
